@@ -271,6 +271,41 @@ def run_all(command, verbose, debug, dry_run, no_lock, update, upgrade, exclude_
         terragrunt_args=terragrunt_args,
     )
 
+@main.command(
+    name="show",
+    context_settings=dict(
+        ignore_unknown_options=True,
+    ),
+)
+@click.option('--verbose', '-v', is_flag=True, default=False,
+    help='Verbose printing',
+    show_default=True
+    )
+@click.option('--working-dir', '-w', default=None,
+    help='Working directory, when omitted the current directory is used',
+    )
+@click.option('--json', '-j', is_flag=True, default=False,
+    help='Show output in json format',
+    )
+@click.option('--planfile-dir', '-P', default=None,
+    help='Relative path to directory with plan file (or set TGWRAP_PLANFILE_DIR environment variable), see README for more details',
+    envvar='TGWRAP_PLANFILE_DIR', type=click.Path(),
+    )
+@click.argument('terragrunt-args', nargs=-1, type=click.UNPROCESSED)
+@click.version_option(version=__version__)
+def show(verbose, json, working_dir, planfile_dir, terragrunt_args):
+    """ Reads and outputs a Terraform state or plan file in a human-readable or json form. """
+
+    check_latest_version(verbose)
+
+    tgwrap = TgWrap(verbose=verbose)
+    tgwrap.show(
+        working_dir=working_dir,
+        json=json,
+        planfile_dir=planfile_dir,
+        terragrunt_args=terragrunt_args,
+    )
+
 @main.command(
     name="import",
     context_settings=dict(
@@ -601,7 +636,7 @@ def sync_dir(
     )
 @click.option('--version-tag', '-V',
     help="Version tag, use 'latest' to, well, get the latest version.",
-    required=True, default='latest', show_default=True,
+    required=False, default=None, show_default=False,
     )
 @click.option('--target-stage', '-t',
     multiple=True,
