@@ -17,7 +17,6 @@ import sys
 import subprocess
 import shlex
 import shutil
-import glob
 import re
 import tempfile
 import json
@@ -29,6 +28,7 @@ import click
 import networkx as nx
 import hcl2
 import fnmatch
+import inquirer
 
 from datetime import datetime, timezone
 from .printer import Printer
@@ -93,7 +93,7 @@ class TgWrap():
 
     def _construct_command(self, command, allow_no_run_all, debug, exclude_external_dependencies,
         non_interactive=True, no_auto_approve=True, no_lock=True, update=False, upgrade=False,
-        planfile=None, working_dir=None, limit_parallelism=None, 
+        planfile=None, working_dir=None, limit_parallelism=None,
         include_dirs=[], exclude_dirs=[], terragrunt_args=(), source_module=None):
         """ Constructs the command """
 
@@ -102,7 +102,7 @@ class TgWrap():
             'info': '{base_command} terragrunt-info --terragrunt-non-interactive {update} {upgrade} {common}',
             'plan': '{base_command} {command} --terragrunt-non-interactive  -out={planfile_name} {lock_level} {update} {parallelism} {common}',
             'apply': '{base_command} {command} {non_interactive} {no_auto_approve} {update} {parallelism} {common} {planfile}',
-            'show': '{base_command} {command} --terragrunt-non-interactive {ignore_deps} {update} -json {planfile_name}', # no working dir allowed here!!!
+            'show': '{base_command} {command} --terragrunt-non-interactive {ignore_deps} {planfile_name} {common}',
             'destroy': '{base_command} {command} {non_interactive} {no_auto_approve} {parallelism} {common} {planfile}',
         }
 
@@ -168,7 +168,11 @@ class TgWrap():
             include_dir_stmt  = ''
             exclude_dir_stmt  = ''
 
-        common_commands = f"{ignore_deps_stmt} {debug_stmt} {working_dir_stmt} {include_dir_stmt} {exclude_dir_stmt} {' '.join(terragrunt_args)}"
+        tg_args_statement = ''
+        if terragrunt_args:
+            tg_args_statement = ' '.join(terragrunt_args)
+
+        common_commands = f"{ignore_deps_stmt} {debug_stmt} {working_dir_stmt} {include_dir_stmt} {exclude_dir_stmt} {tg_args_statement}"
 
         if command not in ['clean']:
             full_command = commands.get(command, commands.get('generic')).format(
@@ -184,6 +188,7 @@ class TgWrap():
                 planfile=planfile_stmt,
                 common=common_commands,
                 planfile_name=self.PLANFILE_NAME,
+                # tg_args=tg_args_statement,
             )
         else:
             full_command = commands.get(command, commands.get('generic'))
@@ -358,43 +363,81 @@ class TgWrap():
     def _clone_repo(self, manifest, target_dir, version_tag=None):
         """Clones the repo, possibly a specific version, into a temp directory"""
 
-        def check_version_tag(reference, working_dir):
-            is_branch = False
-            quiet_mode = "" if self.printer.print_verbose else "--quiet"
-
-            # Check if the given reference is a tag
-            tag_command = f'git show-ref --verify {quiet_mode} refs/tags/{reference}'
-            tag_process = subprocess.run(
-                shlex.split(tag_command),
-                cwd=working_dir,
-                capture_output=True,
+        def get_tags(target_dir):
+            # Run the git command to fetch tags
+            cmd = "git fetch --tags"
+            subprocess.run(
+                shlex.split(cmd),
+                check=True,
+                cwd=target_dir,
+                stdout=sys.stdout if self.printer.print_verbose else subprocess.DEVNULL,
+                stderr=sys.stderr if self.printer.print_verbose else subprocess.DEVNULL,
                 )
-            is_tag = tag_process.returncode == 0
-            self.printer.verbose(f'Check for tag: {tag_process}')
 
-            # if it is not a tag, then it might be a branch
-            if not is_tag:
-                branch_command = f'git switch {reference}'
-                branch_process = subprocess.run(
-                    shlex.split(branch_command),
+            # Run the git command to get the list of tags
+            cmd = "git tag -l --sort=-committerdate"
+            result = subprocess.run(
+                shlex.split(cmd),
+                text=True,
+                check=True,
+                cwd=target_dir,
+                stdout=subprocess.PIPE,
+                stderr=sys.stderr if self.printer.print_verbose else subprocess.DEVNULL,
+                )
+            self.printer.verbose(rc)
+
+            # Get the list of tags
+            tags = result.stdout.strip().split('\n')
+            # remove empty strings
+            tags = [tag for tag in tags if tag != ""]
+
+            tags.insert(0, self.LATEST_VERSION)
+
+            return tags
+
+        def check_version_tag(reference, working_dir):
+            is_latest = (reference == self.LATEST_VERSION)
+            is_branch = False
+            is_tag = False
+
+            if not is_latest:
+                quiet_mode = "" if self.printer.print_verbose else "--quiet"
+
+                # Check if the given reference is a tag
+                tag_command = f'git show-ref --verify {quiet_mode} refs/tags/{reference}'
+                tag_process = subprocess.run(
+                    shlex.split(tag_command),
                     cwd=working_dir,
                     capture_output=True,
                     )
-                is_branch = branch_process.returncode == 0
-                self.printer.verbose(f'Check for branch: {branch_process}')
+                is_tag = tag_process.returncode == 0
+                self.printer.verbose(f'Check for tag: {tag_process}')
+
+                # if it is not a tag, then it might be a branch
+                if not is_tag:
+                    branch_command = f'git switch {reference}'
+                    branch_process = subprocess.run(
+                        shlex.split(branch_command),
+                        cwd=working_dir,
+                        capture_output=True,
+                        )
+                    is_branch = branch_process.returncode == 0
+                    self.printer.verbose(f'Check for branch: {branch_process}')
 
             # Print the result
-            if is_branch:
+            if is_latest:
+                self.printer.verbose(f"The given reference '{reference}' is the latest version.")
+            elif is_branch:
                 self.printer.verbose(f"The given reference '{reference}' is a branch.")
             elif is_tag:
                 self.printer.verbose(f"The given reference '{reference}' is a tag.")
             else:
-                msg = f"The given reference '{reference}' is neither a branch nor a tag."
-                self.printer.verbose(f"The given reference '{reference}' is neither a branch nor a tag.")
+                msg = f"The given reference '{reference}' is neither latest, a branch nor a tag."
+                self.printer.verbose(msg)
                 raise Exception(msg)
                 
 
-            return is_branch, is_tag
+            return is_latest, is_branch, is_tag
 
         # clone the repo
         repo = manifest['git_repository']
@@ -409,28 +452,45 @@ class TgWrap():
             )
         self.printer.verbose(rc)
 
-        # now check out the specific version if we don't want latest
-        if version_tag and version_tag.lower() != self.LATEST_VERSION:
-            # first check if we have a tag or a branch
-            is_branch, is_tag = check_version_tag(
-                reference=version_tag,
-                working_dir=target_dir,
-                )
+        # if we don't have a version number specified, get them and let the user pick one
+        if not version_tag:
+            tags = get_tags(target_dir)
+            questions = [
+                inquirer.List('version',
+                    message="Which version do you want to deploy?",
+                    choices=tags,
+                ),
+            ]
+            version_tag = inquirer.prompt(questions)['version']
 
-            if is_tag:
-                cmd = f"git checkout -b source {version_tag}"
-                rc = subprocess.run(
-                    shlex.split(cmd),
-                    cwd=target_dir,
-                    check=True,
-                    stdout=sys.stdout if self.printer.print_verbose else subprocess.DEVNULL,
-                    stderr=sys.stderr if self.printer.print_verbose else subprocess.DEVNULL,
-                    )
-                self.printer.verbose(rc)
-            elif is_branch:
-                pass # should already be present
-            else:
-                self.printer.error(f'Version tag {version_tag} seems neither a branch or a tag, cannot switch to it!')
+            self.printer.normal
+
+        # first check if we have a tag or a branch
+        is_latest, is_branch, is_tag = check_version_tag(
+            reference=version_tag,
+            working_dir=target_dir,
+            )
+
+        self.printer.header(f'Deploy using reference {version_tag}')
+
+        if is_latest:
+            pass # nothing to do, we already have latest
+        elif is_tag:
+            cmd = f"git checkout -b source {version_tag}"
+            rc = subprocess.run(
+                shlex.split(cmd),
+                cwd=target_dir,
+                check=True,
+                stdout=sys.stdout if self.printer.print_verbose else subprocess.DEVNULL,
+                stderr=sys.stderr if self.printer.print_verbose else subprocess.DEVNULL,
+                )
+            self.printer.verbose(rc)
+        elif is_branch:
+            pass # should already be present
+        else:
+            self.printer.error(f'Version tag {version_tag} seems neither a branch or a tag, cannot switch to it!')
+
+        return version_tag, is_branch, is_tag
 
     def _analyze_results(self, rc, messages):
         """ Checks for errors """
@@ -771,6 +831,62 @@ class TgWrap():
 
             sys.exit(rc.returncode)
 
+    def show(self, working_dir, json, planfile_dir, terragrunt_args):
+        """ Reads and outputs a Terraform state or plan file in a human-readable or json form. """
+
+        # this method is implemented seperatly (and not as part of the run command) to allow
+        # for using native terraform, which speeds up things significantly.
+        # However, for run-all, the regular method is used as changing that has too much impact.
+
+        self.printer.verbose(f"Attempting to run 'show' command using json={json} and planfile_dir={planfile_dir}")
+        if terragrunt_args:
+            self.printer.verbose(f"- with additional parameters: {' '.join(terragrunt_args)}")
+
+        cwd = ""
+        if working_dir:
+            cwd = working_dir
+
+        # determine whether we are going to use a native 'terraform show' (faster!) or need 
+        # to use a terragrunt show
+        if not planfile_dir:
+            self.printer.verbose('Use terragrunt for showing plan')
+
+            # some environments raise the error: AttributeError: ‘tuple’ object has no attribute ‘append’
+            # so convert to a list to make it updateable
+            tg_args_list = list(terragrunt_args)
+
+            # first run a 'show' and write output to file
+            if '-json' not in tg_args_list:
+                tg_args_list.append('-json')
+
+            cmd = self._construct_command(
+                command='show',
+                allow_no_run_all=True,
+                exclude_external_dependencies=True,
+                debug=False,
+                terragrunt_args=tg_args_list,
+            )
+        else:
+            cwd = os.path.join(cwd, planfile_dir)
+
+            json_stmt = ""            
+            if json:
+                json_stmt = "-json"
+
+            self.printer.verbose('Use native terraform for module selection')
+            cmd = f"terraform show {json_stmt} {self.PLANFILE_NAME}"
+
+        # the `posix=False` is to prevent the split command to remove quotes from strings,
+        # e.g. when executing commands like this:
+        # tgwrap state mv 'azuread_group.this["viewers"]' 'azuread_group.this["readers"]'
+        rc = subprocess.run(
+            shlex.split(cmd, posix=False),
+            cwd=cwd,
+            )
+        self.printer.verbose(rc)
+
+        sys.exit(rc.returncode)
+
     def run_all(self, command, debug, dry_run, no_lock, update, upgrade,
         exclude_external_dependencies, step_by_step, planfile, auto_approve, clean,
         working_dir, start_at_step, limit_parallelism, include_dirs, exclude_dirs, terragrunt_args):
@@ -894,13 +1010,21 @@ class TgWrap():
             self.printer.verbose('Use terragrunt for module selection')
 
             use_native_terraform = False
+            
+            # some environments raise the error: AttributeError: ‘tuple’ object has no attribute ‘append’
+            # so convert to a list to make it updateable
+            tg_args_list = list(terragrunt_args)
+
             # first run a 'show' and write output to file
+            if '-json' not in tg_args_list:
+                tg_args_list.append('-json')
+
             cmd = self._construct_command(
                 command='show',
                 allow_no_run_all=True,
                 exclude_external_dependencies=True,
                 debug=False,
-                terragrunt_args=terragrunt_args,
+                terragrunt_args=tg_args_list,
             )
         else:
             self.printer.verbose('Use native terraform for module selection')
@@ -977,6 +1101,19 @@ class TgWrap():
         finally:
             os.remove(f.name)
 
+        # calulate the total drifts and scoe
+        total_drifts = {
+            "creations": 0,
+            "updates": 0,
+            "deletions": 0,
+            "minor": 0,
+            "medium": 0,
+            "major": 0,
+            "unknown": 0,
+            "total": 0,
+            "score": 0,
+        }
+
         self.printer.header("Analysis results:", print_line_before=True)
         for key, value in changes.items():
             self.printer.header(f'Module: {key}')
@@ -989,26 +1126,19 @@ class TgWrap():
             if value["deletions"]:
                 self.printer.warning('Deletions:')
                 for m in value["deletions"]:
+                    total_drifts["deletions"] = total_drifts["deletions"] + 1
                     self.printer.warning(f'-> {m}')
             if value["creations"]:
                 self.printer.normal('Creations:')
                 for m in value["creations"]:
+                    total_drifts["creations"] = total_drifts["creations"] + 1
                     self.printer.normal(f'-> {m}')
             if value["updates"]:
                 self.printer.normal('Updates:')
                 for m in value["updates"]:
+                    total_drifts["updates"] = total_drifts["updates"] + 1
                     self.printer.normal(f'-> {m}')
 
-        # calulate the total drifts and scoe
-        total_drifts = {
-            "minor": 0,
-            "medium": 0,
-            "major": 0,
-            "unknown": 0,
-            "total": 0,
-            "score": 0,
-        }
-
         for key, value in changes.items():
             for type in ["minor", "medium", "major", "unknown", "total"]:
                 total_drifts[type] += value["drifts"][type]
@@ -1033,8 +1163,6 @@ class TgWrap():
                 for m in value["unknowns"]:
                     self.printer.warning(f' -> {m}')
 
-
-
         if out:
             # in the output we convert the dict of dicts to a list of dicts as it makes processing
             # (e.g. by telegraph) easier.
@@ -1063,6 +1191,8 @@ class TgWrap():
 
         command = "destroy" if lock_status == "unlock" else "apply" 
 
+        self.printer.normal(f"Run '{command}' on module {module}")
+
         self.run(
             command=command,
             dry_run=dry_run,
@@ -1072,6 +1202,7 @@ class TgWrap():
             clean=True,
             no_lock=False,
             update=False,
+            upgrade=False,
             planfile=None,
             terragrunt_args=[],
             )
@@ -1160,7 +1291,7 @@ class TgWrap():
             except KeyError:
                 source_config_dir = None
 
-            self._clone_repo(
+            version_tag, _, _ = self._clone_repo(
                 manifest=manifest,
                 target_dir=temp_dir,
                 version_tag=version_tag,
@@ -1466,7 +1597,10 @@ Note:
     def clean(self, working_dir):
         """ Clean the temporary files of a terragrunt/terraform project """
 
-        cmd = 'find . -name ".terragrunt-cache" -type d -exec rm -rf {} \; ; find . -name ".terraform" -type d -exec rm -rf {} \;'
+        cmd = 'find . -name ".terragrunt-cache" -type d -exec rm -rf {} \; ; ' + \
+            'find . -name ".terraform" -type d -exec rm -rf {} \; ; ' + \
+            'find . -name "terragrunt-debug*" -type f -exec rm -rf {} \;'
+
         # we see the behaviour that with cleaning up large directories, it returns errorcode=1 upon first try
         # never to shy away from a questionable solution to make your life easier, we just run it again :-)
         rc = 'clean up not started!'
@@ -1507,12 +1641,16 @@ Note:
 
         current_release = None
         for entry in commit_entries:
-            # Check if the entry contains a release tag
+            # Check if the entry contains a release tag, it has a format like this:
+            # 80cbe7a (HEAD -> main, tag: v2023.11.27.1, origin/main, origin/HEAD) Update readme
             match = re.search(release_pattern, entry)
             if match:
                 current_release = match.group(1)
                 if current_release not in release_commits:
-                    release_commits[current_release] = []
+                    # remove the part between ()
+                    pattern = re.compile('\(.*?\) ')
+                    updated_entry = pattern.sub('', entry)
+                    release_commits[current_release] = [updated_entry]
             elif current_release:
                 release_commits[current_release].append(entry)
         # Print the grouped commits to the console (you can write them to a file as needed)
